--
-- add specialization to all mods.
--
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @edit:		akuenzi
-- @website:	www.Xentro.se
-- @history:	v2.0  - 2017-05-05 - 
-- 				v1.53 - 2014-11-11 - Improvement
-- 				v1.52 - 2013-10-29 - 
--

AddSpecialization = {};
AddSpecialization.proceedWithLoading = false;
AddSpecialization.g_currentModDirectory = g_currentModDirectory;

addModEventListener(AddSpecialization);


-- Configuration
local specializationToAdd = {};

local entry = {};
entry.specialization  	= "SelfPropelledWindrower"; 			-- Name of specialization
entry.className  		= "SelfPropelledWindrower"; 			-- Class name of specialization
entry.filename  		= "SelfPropelledWindrower"; 			-- file name of specialization
entry.l10nNameTag		= ""; 									-- Make text global so we can use it on the mod. Name it like following in modDesc GAME_EXTENSION_01, GAME_EXTENSION_02... 
entry.vehicleTypeLimit 	= { 								-- Limit to what vehicle type to add specialization to, will speed up the process big time but its only optional, can be left empty.
	-- VehicleType, ALLOW
	-- tractor = true
};
entry.specLimit 	  	= { 								-- Limit adding to must have specialization or cant have specialization - Only global scripts (default scripts or global mods)
	{"mower", true}
	-- NAME, REQUIRED / CANT HAVE (True / False)
	-- {"drivable", true} 					
};
entry.searchWords  		= {									-- Limit adding to must have specialization or cant have specialization - This will only check custom specialization's in the vehicleType!
	-- NAME, REQUIRED / CANT HAVE (True / False)
	-- {"somethingOkay" , false}
};

table.insert(specializationToAdd, entry);


for i, ss in ipairs(specializationToAdd) do
	table.insert(ss.specLimit,   {ss.specialization, false});
	table.insert(ss.searchWords, {ss.specialization, false});
	
	if SpecializationUtil.specializations[ss.specialization] == nil then
		SpecializationUtil.registerSpecialization(ss.specialization, ss.className, g_currentModDirectory .. ss.filename .. ".lua");
		AddSpecialization.proceedWithLoading = true;
	else
		print("Add Specialization (" .. ss.specialization .. ") - Error: Specialization have been loaded already by another mod! This process will stop now.");
		table.remove(specializationToAdd, i);
	end;
end;

function AddSpecialization:loadMap(name)	
	-- This is only called once for FS17 so no additional code needed as with older versions.
	if AddSpecialization.proceedWithLoading then
		AddSpecialization:add();
	end;
end;

function AddSpecialization:deleteMap()
end;

function AddSpecialization:mouseEvent(posX, posY, isDown, isUp, button)
end;

function AddSpecialization:keyEvent(unicode, sym, modifier, isDown)
end;

function AddSpecialization:update(dt)
end;

function AddSpecialization:draw()
end;

function AddSpecialization:add()
	for _, ss in ipairs(specializationToAdd) do
		local printHeader = "Add Specialization (" .. ss.specialization .. ") - Info: ";
		local vtIsEmpty = true;
		local vtLetNoneFalsePass = false;
		
		for name, v in pairs(ss.vehicleTypeLimit) do
			vtIsEmpty = false;
			
			if not v then
				vtLetNoneFalsePass = true; -- If only false then let all types which havent been setup pass.
			else
				vtLetNoneFalsePass = false; -- If we got both True/False then only allow those that are set to true.
				break;
			end;
		end;
		
		for vehicleType, vehicle in pairs(VehicleTypeUtil.vehicleTypes) do
			if vehicle ~= nil then
				local vt = Utils.splitString(".", vehicleType);
				local vtCheck = vehicleType;
				
				-- We are an mod!
				if #vt > 1 then
					vtCheck = vt[2];
				end;
				
				-- VehicleType search is empty, let it pass
				if (vtIsEmpty 
				-- Check True / False state and let pass according
				or not vtLetNoneFalsePass and ss.vehicleTypeLimit[vtCheck] ~= nil and ss.vehicleTypeLimit[vtCheck]
				-- Only false vehicleType's have been set, let all which arent set to False pass
				or vtLetNoneFalsePass and ss.vehicleTypeLimit[vtCheck] ~= nil and ss.vehicleTypeLimit[vtCheck]
				or vtLetNoneFalsePass and ss.vehicleTypeLimit[vtCheck] == nil) then
					local forceStop = false;
					local numLimit = {0, 0}; -- Required global scripts found, Must have x specs 
					local errorHelp = {};
					
					-- Global script check
					for _, sl in ipairs(ss.specLimit) do
						if sl[2] then
							numLimit[2] = numLimit[2] + 1;
						end;
						
						for _, vs in ipairs(vehicle.specializations) do
							if vs == SpecializationUtil.getSpecialization(sl[1]) then
								if sl[2] then
									errorHelp[sl[1]] = true;
									numLimit[1] = numLimit[1] + 1;
								else
									-- print(printHeader .. "Unallowed specialization (" .. sl[1] .. ") was found in " .. vehicleType);
									forceStop = true;
									break;
								end;
							end;
						end;
						
						if forceStop then
							break;
						end;
					end;
					
					-- Do name check of custom specializations
					if not forceStop then
						local specializationToSearch = {};
						
						-- Gather all scripts used for vehicle, none global.
						for name in pairs(SpecializationUtil.specializations) do
							if string.find(name, vt[1]) ~= nil then
								local parts = Utils.splitString(".", name);
								
								if #parts > 1 then
									table.insert(specializationToSearch, parts);
								end;
							end;
						end;
						
						-- Process the result 
						for _, sw in ipairs(ss.searchWords) do
							if sw[2] then
								numLimit[2] = numLimit[2] + 1;
							end;
							
							for _, s in ipairs(specializationToSearch) do
								local stringMatch = string.find(string.lower(s[2]), string.lower(sw[1])) ~= nil;
								
								if stringMatch then
									if not sw[2] then
										print(printHeader .. "Unallowed specialization (" .. s[2] .. ") was found in " .. vehicleType);
										forceStop = true;
										break;
									else
										-- We need to count to make sure all required scripts are found.
										numLimit[1] = numLimit[1] + 1;
										errorHelp[sw[1]] = s[2];
									end;
								end;
							end;
							
							if forceStop then
								break;
							end;
						end;
					end;
					
					-- Check if all required scripts was found.
					if numLimit[1] ~= numLimit[2] then
						-- print(printHeader .. "Not all required specialization's was found (" .. numLimit[1] .. "/" .. numLimit[2] .. ") in " .. vehicleType);
						-- print(printHeader .. "List of required specialization's");
						
						for _, sl in ipairs(ss.specLimit) do
							if sl[2] then
								-- print(printHeader .. sl[1] .. " was found: " .. tostring(errorHelp[sl[1]] ~= nil));
							end;
						end;
						
						for _, sw in ipairs(ss.searchWords) do
							if sw[2] then
								-- print(printHeader .. sw[1] .. " was found: " .. tostring(errorHelp[sw[1]] ~= nil) .. " (" .. tostring(errorHelp[sw[1]]) ..")");
							end;
						end;
						
						forceStop = true;
					end;
					
					-- Finish, add script.
					if not forceStop then
						table.insert(vehicle.specializations, SpecializationUtil.getSpecialization(ss.specialization));
						--print(printHeader .. "Inserted on " .. vehicleType);
					end;
				end;
			end;
		end;
		
		-- make l10n global 
		local i = 1;
		while true do
			local txt = string.format(ss.l10nNameTag .. "_%d", i);
			
			if not g_i18n:hasText(txt) then
				break;
			end;
			
			g_i18n.globalI18N.texts[txt] = g_i18n:getText(txt);
			
			i = i + 1;
		end;
		
		-- Added code to make l10n global
		g_i18n.globalI18N.texts["input_GRASSEED_WINDROW"] = g_i18n:getText("input_GRASSEED_WINDROW");
		g_i18n.globalI18N.texts["GRASSEED_WINDROW_ON"] = g_i18n:getText("GRASSEED_WINDROW_ON");
		g_i18n.globalI18N.texts["GRASSEED_WINDROW_OFF"] = g_i18n:getText("GRASSEED_WINDROW_OFF");
		g_i18n.globalI18N.texts["GRASSEED_SHATTERING"] = g_i18n:getText("GRASSEED_SHATTERING");
	end;
end;